# Generated by roxygen2: do not edit by hand

export(bray)
export(canberra)
export(colormap)
export(column_exists)
export(composition_plot)
export(cosine)
export(diversity)
export(diversity_plot)
export(euclidean)
export(hill_taxa)
export(jaccard)
export(jsd)
export(manhattan)
export(matrix_to_dtable)
export(metagenomics)
export(omics)
export(ordination_plot)
export(pairwise_adonis)
export(pairwise_anosim)
export(plot_pairwise_stats)
export(proteomics)
export(unifrac)
export(volcano_plot)
import(Matrix)
import(R6)
import(rhdf5)
importFrom(Matrix,sparseMatrix)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rcpp,sourceCpp)
importFrom(RcppParallel,setThreadOptions)
importFrom(ape,keep.tip)
importFrom(ape,read.tree)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,stat_ellipse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(jsonlite,read_json)
importFrom(jsonlite,toJSON)
importFrom(jsonvalidate,json_validate)
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(stats,as.dist)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,p.adjust.methods)
importFrom(stats,quantile)
importFrom(tools,file_ext)
importFrom(utils,globalVariables)
importFrom(yyjsonr,validate_json_file)
useDynLib(OmicFlow, .registration = TRUE)
