# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

bray <- function(mat) {
    .Call(`_OmicFlow_bray`, mat)
}

euclidean <- function(mat) {
    .Call(`_OmicFlow_euclidean`, mat)
}

cosine <- function(mat) {
    .Call(`_OmicFlow_cosine`, mat)
}

jaccard <- function(mat) {
    .Call(`_OmicFlow_jaccard`, mat)
}

manhattan <- function(mat) {
    .Call(`_OmicFlow_manhattan`, mat)
}

canberra <- function(mat) {
    .Call(`_OmicFlow_canberra`, mat)
}

jsd <- function(mat) {
    .Call(`_OmicFlow_jsd`, mat)
}

unifrac <- function(mat, edge, edge_lengths, weighted, normalized) {
    .Call(`_OmicFlow_unifrac`, mat, edge, edge_lengths, weighted, normalized)
}

