# Testing Proteomics loading

    Code
      counts_without_rownames
    Message
      
      -- <proteomics> object 
      metaData: 2 variables x 10 samples
      countData: 10 samples x 50 features
      featureData: 0 attributes x 50 features

---

    Code
      counts_with_rownames
    Message
      
      -- <proteomics> object 
      metaData: 2 variables x 10 samples
      countData: 10 samples x 50 features
      featureData: 0 attributes x 50 features
      treeData: 50 tips x 49 nodes

---

    Code
      prot_data
    Message
      
      -- <proteomics> object 
      metaData: 2 variables x 10 samples
      countData: 10 samples x 50 features
      featureData: 0 attributes x 50 features
      treeData: 50 tips x 49 nodes

