# Testing Compositional data

    Code
      res$data
    Output
      Key: <SAMPLE_ID>
          SAMPLE_ID                   Genus      value CONTRAST_sex
             <char>                  <fctr>      <num>       <char>
       1:      S100         Woesearchaeales 0.02022059         male
       2:      S100            Sideroxydans 0.19209559         male
       3:      S100           Acinetobacter 0.16727941         male
       4:      S100  Candidatus_Omnitrophus 0.05147059         male
       5:      S100             Pseudomonas 0.09099265         male
       6:      S100          Nitrosarchaeum 0.06341912         male
       7:      S100 Candidatus_Peribacteria 0.00000000         male
       8:      S100              Nitrospira 0.03033088         male
       9:      S100           Sulfurifustis 0.00000000         male
      10:      S100           Aquabacterium 0.03400735         male
      11:      S100                   Other 0.09466912         male
      12:      S103         Woesearchaeales 0.28865979       female
      13:      S103            Sideroxydans 0.00000000       female
      14:      S103           Acinetobacter 0.02268041       female
      15:      S103  Candidatus_Omnitrophus 0.06391753       female
      16:      S103             Pseudomonas 0.00000000       female
      17:      S103          Nitrosarchaeum 0.00000000       female
      18:      S103 Candidatus_Peribacteria 0.04123711       female
      19:      S103              Nitrospira 0.00000000       female
      20:      S103           Sulfurifustis 0.03402062       female
      21:      S103           Aquabacterium 0.00000000       female
      22:      S103                   Other 0.20618557       female
      23:      S115         Woesearchaeales 0.03149002       female
      24:      S115            Sideroxydans 0.20890937       female
      25:      S115           Acinetobacter 0.09677419       female
      26:      S115  Candidatus_Omnitrophus 0.08986175       female
      27:      S115             Pseudomonas 0.04685100       female
      28:      S115          Nitrosarchaeum 0.09831029       female
      29:      S115 Candidatus_Peribacteria 0.00000000       female
      30:      S115              Nitrospira 0.03917051       female
      31:      S115           Sulfurifustis 0.01228879       female
      32:      S115           Aquabacterium 0.02534562       female
      33:      S115                   Other 0.14669739       female
      34:      S120         Woesearchaeales 0.12540894         male
      35:      S120            Sideroxydans 0.00000000         male
      36:      S120           Acinetobacter 0.00000000         male
      37:      S120  Candidatus_Omnitrophus 0.06434024         male
      38:      S120             Pseudomonas 0.05779716         male
      39:      S120          Nitrosarchaeum 0.00000000         male
      40:      S120 Candidatus_Peribacteria 0.05125409         male
      41:      S120              Nitrospira 0.01853871         male
      42:      S120           Sulfurifustis 0.03707743         male
      43:      S120           Aquabacterium 0.00000000         male
      44:      S120                   Other 0.20828790         male
          SAMPLE_ID                   Genus      value CONTRAST_sex
             <char>                  <fctr>      <num>       <char>

---

    Code
      res$palette
    Output
              Woesearchaeales            Sideroxydans           Acinetobacter 
                    "#A50026"               "#D73027"               "#F46D43" 
       Candidatus_Omnitrophus             Pseudomonas          Nitrosarchaeum 
                    "#FDAE61"               "#FEE090"               "#E0F3F8" 
      Candidatus_Peribacteria              Nitrospira           Sulfurifustis 
                    "#ABD9E9"               "#74ADD1"               "#4575B4" 
                Aquabacterium                   Other 
                    "#313695"             "lightgrey" 

