# Testing omics loading from existing data objects

    Code
      ends_with_featureData
    Message
      
      -- <omics> object 
      metaData: 1 variables x 5 samples
      countData: 5 samples x 100 features
      featureData: 0 attributes x 100 features

---

    Code
      ends_with_countData
    Message
      
      -- <omics> object 
      metaData: 1 variables x 5 samples
      countData: 5 samples x 100 features
      featureData: 0 attributes x 100 features

