# Testing omics loading from file

    Code
      with_features_sparse
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features

---

    Code
      with_features_dense
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features

---

    Code
      without_features_with_rownames
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 0 attributes x 242 features

---

    Code
      without_features_without_rownames
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 0 attributes x 242 features

