# Testing subsetting of data

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 185 features
      featureData: 7 attributes x 185 features
      treeData: 185 tips x 184 nodes

---

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 2 samples
      countData: 2 samples x 115 features
      featureData: 7 attributes x 115 features
      treeData: 115 tips x 114 nodes

---

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 200 features
      featureData: 7 attributes x 200 features
      treeData: 200 tips x 199 nodes

---

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 50 features
      featureData: 7 attributes x 50 features
      treeData: 50 tips x 49 nodes

