# Testing Metagenomics reading and writing of BIOM files

    Code
      taxa_hdf5
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features
      treeData: 242 tips x 241 nodes

---

    Code
      taxa_ref
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features
      treeData: 242 tips x 241 nodes

---

    Code
      taxa_json
    Message
      
      -- <metagenomics> object 
      metaData: 1 variables x 6 samples
      countData: 6 samples x 5 features
      featureData: 8 attributes x 5 features

