% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{column_exists}
\alias{column_exists}
\title{Checks if column exists in table}
\usage{
column_exists(column, table)
}
\arguments{
\item{column}{A character of length 1.}

\item{table}{A \link[data.table]{data.table} or \link[base]{data.frame}.}
}
\value{
A boolean value.
}
\description{
Mainly used within \link{omics} and other functions to check if given column name does exist in the table and is not completely empty (containing NAs).
}
