% Generated by roxygen2: do not edit by hand % Please edit documentation in R/proteomics-class.R \name{proteomics} \alias{proteomics} \title{Sub-class proteomics} \description{ This is a sub-class that is compatible to preprocessed data obtained from https://fragpipe.nesvilab.org/. It inherits all methods from the abstract class \link{omics} and only adapts the \code{initialize} function. It supports pre-existing data structures or paths to text files. When omics data is very large, data loading becomes very expensive. It is therefore recommended to use the \href{#method-reset}{\code{reset()}} method to reset your changes. Every omics class creates an internal memory efficient back-up of the data, the resetting of changes is an instant process. } \seealso{ \link{omics} } \section{Super class}{ \code{\link[OmicFlow:omics]{OmicFlow::omics}} -> \code{proteomics} } \section{Active bindings}{ \if{html}{\out{
}} \describe{ \item{\code{treeData}}{A "phylo" class, see \link[ape]{as.phylo}.} } \if{html}{\out{
}} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-proteomics-new}{\code{proteomics$new()}} \item \href{#method-proteomics-clone}{\code{proteomics$clone()}} } } \if{html}{\out{
Inherited methods

- `OmicFlow::omics$alpha_diversity()`
- `OmicFlow::omics$autoFlow()`
- `OmicFlow::omics$composition()`
- `OmicFlow::omics$copy()`
- `OmicFlow::omics$distance()`
- `OmicFlow::omics$feature_merge()`
- `OmicFlow::omics$feature_subset()`
- `OmicFlow::omics$foldchange()`
- `OmicFlow::omics$ordination()`
- `OmicFlow::omics$print()`
- `OmicFlow::omics$rankstat()`
- `OmicFlow::omics$removeNAs()`
- `OmicFlow::omics$reset()`
- `OmicFlow::omics$sample_subset()`
- `OmicFlow::omics$samplepair_subset()`
- `OmicFlow::omics$scale()`
- `OmicFlow::omics$validate()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-proteomics-new}{}}} \subsection{Method \code{new()}}{ Initializes the proteomics class object with \code{proteomics$new()} \subsection{Usage}{ \if{html}{\out{
}}\preformatted{proteomics$new( countData = NULL, metaData = NULL, featureData = NULL, treeData = NULL )}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{countData}}{A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.} \item{\code{metaData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.} \item{\code{featureData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.} \item{\code{treeData}}{A path to an existing newick file or class "phylo", see \link[ape]{read.tree}.} } \if{html}{\out{
}} } \subsection{Returns}{ A new \code{proteomics} object. } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-proteomics-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{proteomics$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }