% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{matrix_to_dtable}
\alias{matrix_to_dtable}
\title{Converting a Matrix to data.table}
\usage{
matrix_to_dtable(x)
}
\arguments{
\item{x}{A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.}
}
\value{
A \link[data.table]{data.table} class.
}
\description{
Wrapper function that converts a sparseMatrix to data.table
}
