% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OmicFlow-package.R
\docType{package}
\name{OmicFlow-package}
\alias{OmicFlow}
\alias{OmicFlow-package}
\title{OmicFlow: Fast and Efficient (Automated) Analysis of Sparse Omics Data}
\description{
A generalised data structure for fast and efficient loading and data munching of sparse omics data. The 'OmicFlow' requires an up-front validated metadata template from the user, which serves as a guide to connect all the pieces together by aligning them into a single object that is defined as an 'omics' class. Once this unified structure is established, users can perform manual subsetting, visualisation, and statistical analysis, or leverage the automated 'autoFlow' method to generate a comprehensive report.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/agusinac/OmicFlow}
  \item Report bugs at \url{https://github.com/agusinac/OmicFlow/issues}
}

}
\author{
\strong{Maintainer}: Alem Gusinac \email{alem.gusinac@gmail.com} (\href{https://orcid.org/0009-0006-1896-4176}{ORCID})

Authors:
\itemize{
  \item Thomas Ederveen (\href{https://orcid.org/0000-0003-0068-1275}{ORCID})
  \item Annemarie Boleij (\href{https://orcid.org/0000-0003-4495-5880}{ORCID}) [funder]
}

}
\keyword{internal}
